#!/usr/bin/env python
import argparse
import json
import logging

import paho.mqtt.client as mqtt

from ardurpc import ArduRPC
from ardurpc.connector.serial_con import Serial

con = None
rpc = None
base_topic = None


def on_connect(client, userdata, flags, rc):
    print("Connected with result code " + str(rc))
    # Subscribe to base topic and all sub topics
    client.subscribe(base_topic + "#")


def on_message(client, userdata, msg):
    if not msg.topic.startswith(base_topic):
        logging.warning("Topic not allowed: %s" % msg.topic)
        return

    # Strip base topic
    tmp = msg.topic[len(base_topic):]
    # Split handler and command
    handler, sep, command = tmp.partition("/")

    # Try to find the handle with the given name
    h = rpc.get_handler_by_name(handler)
    if h is None:
        logging.warning("Unable to find handler '%s'" % handler)
        return

    # Check for internal commands
    if command.startswith("_") or command.endswith("_"):
        logging.warning("Command might be an internal Python command")
        return

    # Parse data
    data = []
    if len(msg.payload) > 0:
        try:
            data = json.loads(msg.payload.decode("utf-8"))
        except ValueError as e:
            logging.warning("Unable to parse json data: '%s'" % str(e))
            return

    # Get command function from handler object
    try:
        func = getattr(h, command)
    except AttributeError:
        func = None

    if func is None:
        logging.warning("Unable to find command '%s'" % command)
        return

    # Call function
    try:
        func(*data)
    except TypeError as e:
        logging.warning("Unable to call function: '%s'" % str(e))
        return


def main():
    global base_topic, con, rpc
    # Parse args
    parser = argparse.ArgumentParser(description="ArduRPC MQTT Gateway")
    parser.add_argument(
        "-b",
        "--baud",
        default=9600,
        dest="baud",
        help="Baud",
        metavar="N",
        type=int,
    )
    parser.add_argument(
        "-D",
        "--device",
        default="/dev/ttyUSB0",
        dest="device",
        help="Serial Device",
        type=str,
    )
    parser.add_argument(
        "--host",
        default="localhost",
        dest="hostname",
        help="Hostname of the MQTT Broker",
        type=str,
    )
    parser.add_argument(
        "--port",
        default=1883,
        dest="port",
        help="Port of the MQTT Broker",
        type=int,
    )
    parser.add_argument(
        "--base-topic",
        default="arduino/ardurpc",
        dest="base_topic",
        help="Base topic (Default: arduino/ardurpc)",
        type=str,
    )

    # Parse commandline arguments
    args = parser.parse_args()

    # Connect to Arduino and initialize ArduRPC interface
    con = Serial(args.device, args.baud)
    rpc = ArduRPC(connector=con)
    base_topic = args.base_topic.strip()
    if not base_topic.endswith("/"):
        base_topic += "/"

    # Initialize new MQTT client and connect to broker
    client = mqtt.Client()
    client.on_connect = on_connect
    client.on_message = on_message

    client.connect(args.hostname, args.port, 60)

    client.loop_forever()

if __name__ == "__main__":
    main()